@@ -7,7 +7,7 @@ import random
 import re
 import sys
 from encodings.aliases import aliases as ALIASES
-from functools import reduce, wraps
+from functools import reduce, update_wrapper, wraps
 from importlib import import_module
 from inspect import currentframe
 from itertools import chain, product
@@ -19,28 +19,35 @@ from random import randint
 from six import binary_type, string_types, text_type, BytesIO
 from string import *
 from types import FunctionType, ModuleType
-try: # Python3
-    from importlib import reload
+try:  # Python2
+    import __builtin__ as builtins
 except ImportError:
-    pass
-try:  # Python3
+    import builtins
+try:  # Python2
     from inspect import getfullargspec
 except ImportError:
     from inspect import getargspec as getfullargspec
-try:                 # Python 2
+try:  # Python2
     from string import maketrans
-except ImportError:  # Python 3
+except ImportError:
     maketrans = str.maketrans
+try:  # Python3
+    from importlib import reload
+except ImportError:
+    pass
 
 
 __all__ = ["add", "add_macro", "add_map", "b", "clear", "codecs", "decode", "encode", "ensure_str", "examples", "guess",
-           "isb", "generate_strings_from_regex", "get_alphabet_from_mask", "handle_error", "is_native",
+           "isb", "generate_strings_from_regex", "get_alphabet_from_mask", "handle_error", "i2s", "is_native",
            "list_categories", "list_encodings", "list_macros", "lookup", "maketrans", "os", "rank", "re", "register",
            "remove", "reset", "s2i", "search", "stopfunc", "BytesIO", "_input", "_stripl", "CodecMacro",
            "DARWIN", "LANG", "LINUX", "MASKS", "PY3", "UNIX", "WINDOWS"]
 CODECS_REGISTRY = None
+CODECS_OVERWRITTEN = []
 CODECS_CATEGORIES = ["native", "custom"]
-LANG = getlocale()[0][:2].lower() if getlocale() else None
+LANG = getlocale()
+if LANG:
+    LANG = (LANG[0] or "")[:2].lower()
 MASKS = {
     'a': printable,
     'b': "".join(chr(i) for i in range(256)),
@@ -72,6 +79,12 @@ iss = lambda s: isinstance(s, string_types)
 fix = lambda x, ref: b(x) if isb(ref) else ensure_str(x) if iss(ref) else x
 
 s2i = lambda s: int(codecs.encode(s, "base16"), 16)
+exc_name = lambda e: "".join(t.capitalize() for t in re.split(r"[-_+]", e))
+
+
+def i2s(input):
+    h = hex(input)[2:].rstrip("eL")
+    return codecs.decode(h.zfill(len(h) + len(h) % 2), "hex")
 
 
 class CodecMacro(tuple):
@@ -142,6 +155,20 @@ class CodecMacro(tuple):
         return "<codext.CodecMacro object for encoding %s at %#x>" % (self.name, id(self))
 
 
+# inspired from: https://stackoverflow.com/questions/10875442/possible-to-change-a-functions-repr-in-python
+class Repr(object):
+    def __init__(self, name, func):
+        self.__name = name
+        self.__func = func
+        update_wrapper(self, func)
+    
+    def __call__(self, *args, **kwargs):
+        return self.__func(*args, **kwargs)
+    
+    def __repr__(self):
+        return "<search-function %s at 0x%x>" % (self.__name, id(self))
+
+
 def __stdin_pipe():
     """ Stdin pipe read function. """
     try:
@@ -155,16 +182,24 @@ def __stdin_pipe():
 
 def _input(infile):
     # handle input file or stdin
+    c = b("")
     if infile:
         with open(infile, 'rb') as f:
             c = f.read()
     else:
-        c = b("")
         for line in __stdin_pipe():
             c += line
     return c
 
 
+def _set_exc(name, etype="ValueError"):
+    if not hasattr(builtins, name):
+        exec("class %s(%s): __module__ = 'builtins'" % (name, etype))
+        setattr(builtins, name, locals()[name])
+_set_exc("InputSizeLimitError")
+_set_exc("ParameterError")
+
+
 def _stripl(s, st_lines, st_crlf):
     if st_crlf:
         s = s.replace(b"\r\n", b"") if isb(s) else s.replace("\r\n", "")
@@ -173,6 +208,12 @@ def _stripl(s, st_lines, st_crlf):
     return s
 
 
+def _with_repr(name):
+    def _wrapper(f):
+        return Repr(name, f)
+    return _wrapper
+
+
 def add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=False, **kwargs):
     """ This adds a new codec to the codecs module setting its encode and/or decode functions, eventually dynamically
          naming the encoding with a pattern and with file handling.
@@ -187,14 +228,21 @@ def add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=
                                 to remove the codec later
     """
     remove(ename)
-    if encode and not isinstance(encode, FunctionType):
-        raise ValueError("Bad 'encode' function")
-    if decode and not isinstance(decode, FunctionType):
-        raise ValueError("Bad 'decode' function")
+    if encode:
+        if not isinstance(encode, FunctionType):
+            raise ValueError("Bad 'encode' function")
+        _set_exc("%sEncodeError" % exc_name(ename))  # create the custom encode exception as a builtin
+    if decode:
+        if not isinstance(decode, FunctionType):
+            raise ValueError("Bad 'decode' function")
+        _set_exc("%sDecodeError" % exc_name(ename))  # create the custom decode exception as a builtin
     if not encode and not decode:
         raise ValueError("At least one en/decoding function must be defined")
+    for exc in kwargs.get('extra_exceptions', []):
+        _set_exc(exc)  # create additional custom exceptions as builtins
     glob = currentframe().f_back.f_globals
     # search function for the new encoding
+    @_with_repr(ename)
     def getregentry(encoding):
         if encoding != ename and not (pattern and re.match(pattern, encoding)):
             return
@@ -304,6 +352,7 @@ def add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=
         getregentry.__aliases__ = list(map(lambda n: re.sub(r"[\s\-]", "_", n), kwargs['aliases']))
     getregentry.__pattern__ = pattern
     register(getregentry, add_to_codecs)
+    return getregentry
 
 
 def add_macro(mname, *encodings):
@@ -488,7 +537,8 @@ def add_map(ename, encmap, repl_char="?", sep="", ignore_case=None, no_error=Fal
                         text = "".join(str(ord(c)).zfill(3) for c in text)
                 r = ""
                 lsep = "" if decode else sep if len(sep) <= 1 else sep[0]
-                error_func = handle_error(ename, errors, lsep, repl_char, rminlen, decode)
+                kind = ["character", "token"][tmaxlen > 1]
+                error_func = handle_error(ename, errors, lsep, repl_char, rminlen, decode, kind)
                 
                 # get the value from the mapping dictionary, trying the token with its inverted case if relevant
                 def __get_value(token, position, case_changed=False):
@@ -500,7 +550,7 @@ def add_map(ename, encmap, repl_char="?", sep="", ignore_case=None, no_error=Fal
                             return __get_value(token_inv_case, position, True)
                         return error_func(token, position)
                     if isinstance(result, list):
-                        result = random.choice(result)
+                        result = result[0]
                     return result + lsep
                 
                 # if a separator is defined, rely on it by splitting the input text
@@ -567,7 +617,7 @@ def add_map(ename, encmap, repl_char="?", sep="", ignore_case=None, no_error=Fal
         kwargs['repl_minlen_b'] = max(1, min(map(len, map(b, set(smapdict.values()) - {''}))))
     except:
         pass
-    add(ename, __generic_code(), __generic_code(True), **kwargs)
+    return add(ename, __generic_code(), __generic_code(True), **kwargs)
 codecs.add_map = add_map
 
 
@@ -651,17 +701,15 @@ def list_encodings(*categories):
     if (len(categories) == 0 or "native" in categories) and "native" not in exclude:
         for a in set(ALIASES.values()):
             try:
-                __orig_lookup(a)
+                ci = __orig_lookup(a)
             except LookupError:
                 continue
-            enc.append(a)
-    for search_function in __codecs_registry:
+            if lookup(a) is ci:
+                enc.append(ci.name)
+    for search_function in CODECS_OVERWRITTEN + __codecs_registry:
         name = search_function.__name__.replace("_", "-")
         p = search_function.__pattern__
-        if p is None:
-            ci = search_function(name)
-        else:
-            ci = search_function(generate_string_from_regex(p))
+        ci = search_function(name) if p is None else search_function(generate_string_from_regex(p))
         c = "other" if ci is None else ci.parameters['category']
         if (len(categories) == 0 or c in categories) and c not in exclude:
             enc.append(name)
@@ -696,6 +744,11 @@ def remove(name):
             json.dump(PERS_MACROS, f, indent=2)
     except KeyError:
         pass
+    for s in ["En", "De"]:
+        try:
+            delattr(builtins, "%s%scodeError" % (name.capitalize(), s))
+        except AttributeError:
+            pass
 codecs.remove = remove
 
 
@@ -789,7 +842,7 @@ def get_alphabet_from_mask(mask):
 
 
 # generic error handling function
-def handle_error(ename, errors, sep="", repl_char="?", repl_minlen=1, decode=False, item="position"):
+def handle_error(ename, errors, sep="", repl_char="?", repl_minlen=1, decode=False, kind="character", item="position"):
     """ This shortcut function allows to handle error modes given some tuning parameters.
     
     :param ename:       encoding name
@@ -800,13 +853,9 @@ def handle_error(ename, errors, sep="", repl_char="?", repl_minlen=1, decode=Fal
     :param decode:      whether we are encoding or decoding
     :param item:        position item description (for describing the error ; e.g. "group" or "token")
     """
-    name = "".join(t.capitalize() for t in re.split(r"[-_+]", ename))
-    # dynamically make dedicated exception classes bound to the related codec module
-    exc = "%s%scodeError" % (name, ["En", "De"][decode])
-    glob = {'__name__': "__main__"}
-    exec("class %s(ValueError): pass" % exc, glob)
-    
-    def _handle_error(token, position, output=""):
+    exc = "%s%scodeError" % (exc_name(ename), ["En", "De"][decode])
+     
+    def _handle_error(token, position, output="", eename=None):
         """ This handles an encoding/decoding error according to the selected handling mode.
         
         :param token:    input token to be encoded/decoded
@@ -814,9 +863,11 @@ def handle_error(ename, errors, sep="", repl_char="?", repl_minlen=1, decode=Fal
         :param output:   output, as decoded up to the position of the error
         """
         if errors == "strict":
-            msg = "'{}' codec can't {}code character '{}' in {} {}"
-            err = glob[exc](msg.format(ename, ["en", "de"][decode], token, item, position))
+            msg = "'%s' codec can't %scode %s '%s' in %s %d"
+            token = token[:7] + "..." if len(token) > 10 else token
+            err = getattr(builtins, exc)(msg % (eename or ename, ["en", "de"][decode], kind, token, item, position))
             err.output = output
+            err.__cause__ = err
             raise err
         elif errors == "leave":
             return token + sep
@@ -834,8 +885,9 @@ __orig_lookup   = _codecs.lookup
 __orig_register = _codecs.register
 
 
-def __add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=True):
-    add(ename, encode, decode, pattern, text, add_to_codecs)
+def __add(ename, encode=None, decode=None, pattern=None, text=True, **kwargs):
+    kwargs.pop('add_to_codecs', None)
+    return add(ename, encode, decode, pattern, text, True, **kwargs)
 __add.__doc__ = add.__doc__
 codecs.add = __add
 
@@ -862,19 +914,19 @@ codecs.encode = encode
 def lookup(encoding, macro=True):
     """ Hooked lookup function for searching first for codecs in the local registry of this module. """
     # first, try to match the given encoding with codecs' search functions
-    for search_function in __codecs_registry:
+    for search_function in CODECS_OVERWRITTEN + __codecs_registry:
         codecinfo = search_function(encoding)
         if codecinfo is not None:
             return codecinfo
     # then, if a codec name was given, generate an encoding name from its pattern and get the CodecInfo
-    for search_function in __codecs_registry:
+    for search_function in CODECS_OVERWRITTEN + __codecs_registry:
         if search_function.__name__.replace("_", "-") == encoding or \
            encoding in getattr(search_function, "__aliases__", []):
             codecinfo = search_function(generate_string_from_regex(search_function.__pattern__))
             if codecinfo is not None:
                 return codecinfo
+    # finally, get a CodecInfo with the original lookup function and refine it with a dictionary of parameters
     try:
-        # finally, get a CodecInfo with the original lookup function and refine it with a dictionary of parameters
         ci = __orig_lookup(encoding)
         ci.parameters = {'category': "native", 'module': "codecs", 'name': ALIASES.get(ci.name, ci.name)}
         return ci
@@ -884,7 +936,9 @@ def lookup(encoding, macro=True):
     try:
         return CodecMacro(encoding)
     except LookupError:
-        raise LookupError("unknown encoding: %s" % encoding)
+        e = LookupError("unknown encoding: %s" % encoding)
+        e.__cause__ = e  # stop exception chaining
+        raise e
 codecs.lookup = lookup
 
 
@@ -898,14 +952,19 @@ def register(search_function, add_to_codecs=False):
                                  to remove the codec later
     """
     if search_function not in __codecs_registry:
-        __codecs_registry.append(search_function)
+        try:
+            __orig_lookup(search_function.__name__)
+            l = CODECS_OVERWRITTEN
+        except LookupError:
+            l = __codecs_registry
+        l.append(search_function)
     if add_to_codecs:
         __orig_register(search_function)
 
 
-def __register(search_function, add_to_codecs=True):
+def __register(search_function):
     """ Same as register(...), but with add_to_codecs set by default to True. """
-    register(search_function, add_to_codecs)
+    register(search_function, True)
 codecs.register = __register
 
 
@@ -1095,7 +1154,7 @@ stopfunc.flag.__name__ = stopfunc.flag.__qualname__ = "flag"
 stopfunc.default = stopfunc.text
 
 stopfunc.LANG_BACKEND = None
-stopfunc.LANG_BACKENDS = [n for n in ["langid", "langdetect", "pycld2", "cld3", "textblob"] if __module_exists(n)]
+stopfunc.LANG_BACKENDS = [n for n in ["pycld2", "langdetect", "langid", "cld3", "textblob"] if __module_exists(n)]
 if len(stopfunc.LANG_BACKENDS) > 0:
     stopfunc.LANG_BACKEND = stopfunc.LANG_BACKENDS[0]
 if "cld3" in stopfunc.LANG_BACKENDS:
@@ -1164,7 +1223,6 @@ def _load_lang_backend(backend=None):
             flng = "lang_%s" % LANG
             if getattr(stopfunc, flng, None):
                 stopfunc.default = getattr(stopfunc, flng)        
-_load_lang_backend(stopfunc.LANG_BACKEND)
 stopfunc._reload_lang = _load_lang_backend
 
 
@@ -1185,10 +1243,12 @@ def __develop(encodings):
 
 
 def __guess(prev_input, input, stop_func, depth, max_depth, min_depth, codec_categories, exclude, result, found=(),
-            stop=True, show=False, scoring_heuristic=False, extended=False, debug=False):
+            stop=True, show=False, scoring_heuristic=False, extended=False, debug=False, regex=False):
     """ Perform a breadth-first tree search using a ranking logic to select and prune the list of codecs. """
     if depth > min_depth and stop_func(input):
-        if not stop and show and found not in result:
+        if regex:
+            stop = True
+        if not stop and (show or debug) and found not in result:
             s = repr(input)
             s = s[2:-1] if s.startswith("b'") and s.endswith("'") else s
             s = "[+] %s: %s" % (", ".join(found), s)
@@ -1228,7 +1288,7 @@ def __guess(prev_input, input, stop_func, depth, max_depth, min_depth, codec_cat
         if debug:
             print("[*] Depth %0{}d/%d: %s".format(len(str(max_depth))) % (depth+1, max_depth, encoding))
         __guess(input, new_input, stop_func, depth+1, max_depth, min_depth, codec_categories, exclude, result,
-                found + (encoding, ), stop, show, scoring_heuristic, extended, debug)
+                found + (encoding, ), stop, show, scoring_heuristic, extended, debug, regex)
 
 
 def __rank(prev_input, input, prev_encoding, codecs, heuristic=False, extended=False, yield_score=False):
@@ -1315,7 +1375,7 @@ def __score(prev_input, input, prev_encoding, codec, heuristic=False, extended=F
                     epxf = f - .1 <= expf <= f + .1
                 elif isinstance(expf, (tuple, list)) and len(expf) == 2:
                     expf = f - expf[1] <= expf[0] <= expf[1] + .1
-                s += .1
+                s += [-1., .1][expf]
             # afterwards, if the input text has an entropy close to the expected one, give a bonus weighted on the
             #  number of input characters to take bad entropies of shorter strings into account
             entr = sc.get('entropy', {})
@@ -1349,18 +1409,22 @@ def guess(input, stop_func=stopfunc.default, min_depth=0, max_depth=5, codec_cat
     """ Try decoding without the knowledge of the encoding(s). """
     if max_depth <= 0:
         raise ValueError("Depth must be a non-null positive integer")
+    if min_depth > max_depth:
+        raise ValueError("Min depth shall be less than or equal to the max depth")
     if len(found) > 0:
         for encoding in found:
             input = decode(input, encoding)
+    regex = False
     if isinstance(stop_func, string_types):
         stop_func = stopfunc.regex(stop_func)
+        regex = True
     result = {}
     if len(input) > 0:
         try:
             # breadth-first search
             for d in range(max_depth):
                 __guess("", input, stop_func, 0, d+1, min_depth, codec_categories, exclude, result, tuple(found), stop,
-                        show, scoring_heuristic, extended, debug)
+                        show, scoring_heuristic, extended, debug, regex)
                 if stop and len(result) > 0:
                     return result
         except KeyboardInterrupt:
